#include <Wire.h>
#include <Adafruit_DRV2605.h>

Adafruit_DRV2605 drv;

String inputString = "";
float receivedAngle = 0.0;
bool hasReceivedData = false;

unsigned long lastVibrationTime = 0;
int vibrationInterval = 400;
uint8_t waveformID = 0;

void setup() {
  Serial.begin(9600);
  Wire.begin();

  if (!drv.begin()) {
    Serial.println("DRV2605 not found. Check wiring!");
    while (1);
  }

  drv.selectLibrary(1);  // Use 1 for LRA motor
  drv.setMode(DRV2605_MODE_INTTRIG);
  drv.setWaveform(1, 0);  // Stop on startup

  Serial.println("Ready. Waiting for Unity...");
}

void loop() {
  // 1. Read angle from Unity (line-by-line)
  while (Serial.available()) {
    char c = Serial.read();
    if (c == '\n') {
      float rawAngle = inputString.toFloat();
      inputString = "";

      // ✅ Convert negative angles to positive
      float absAngle = abs(rawAngle);

      // ✅ Clamp to 0–25°
      receivedAngle = constrain(absAngle, 0.0, 25.0);
      hasReceivedData = true;

      Serial.print("Angle received: ");
      Serial.println(receivedAngle);
    } else {
      inputString += c;
    }
  }

  if (!hasReceivedData) return;

  // 2. Map angle to vibration profile
  float angle = receivedAngle;

  if (angle < 3.0) {
    waveformID = 0;
    vibrationInterval = 1000; // No vibration
  } else if (angle <= 8.0) {
    waveformID = 15;
    vibrationInterval = 400;
  } else if (angle <= 13.0) {
    waveformID = 47;
    vibrationInterval = 200;
  } else if (angle <= 19.0) {
    waveformID = 87;
    vibrationInterval = 100;
  } else if (angle <= 25.0) {
    waveformID = 94;
    vibrationInterval = 40;
  }

  // 3. Trigger vibration non-blocking
  if (millis() - lastVibrationTime >= vibrationInterval) {
    if (waveformID != 0) {
      drv.setWaveform(0, waveformID);
      drv.setWaveform(1, 0);
      drv.go();
    }
    lastVibrationTime = millis();
  }
}
